#####################################################################################
## Components of Gamma metric
## Fig. 4, Fig. S7
rm(list=ls(all=TRUE))
library(fields)
library(maps)
library(spatial)
library(maptools)
library(raster)
library(RColorBrewer)
library(Hmisc) # weighted statistics
library(plotrix)
library(stringr)

### filesystem details (paths) removed ###
basepath <- c("***")
climate.scen.path <- "***"
lpj.input.path <- "***"

reference.path <- "***"
scenario.path  <- "***"
metric.path <- "***"

metric.filenames <- list()
metric.filenames[["all"]]=list(deltaV="deltav_combined_metric.data_dec2012.RData",
                               local="all_local_metric.data_dec2012.RData",
                               global="all_global_metric.data_dec2012.RData",
                               balance="all_balance_metric.data_dec2012.RData",
                               combined="all_combined_metric.data_dec2012.RData", # dummy, not used, file actually contains Gamma
                               gamma="all_combined_metric.data_dec2012.RData"
                               )
metric.filenames[["cstocks"]]=list(combined="cstocks_combined_metric.data_dec2012.RData")
metric.filenames[["cfluxes"]]=list(combined="cfluxes_combined_metric.data_dec2012.RData")
metric.filenames[["wfluxes"]]=list(combined="wfluxes_combined_metric.data_dec2012.RData")

force_reload <- FALSE
save_statistics <- TRUE


print("Load biome classification derived from LPJ reference run")
biome.filename <- paste(basepath, reference.path, "biome_class.bin", sep="")
biome.lpj <- readBin(biome.filename, what=integer(), size=1, n=file.info(biome.filename)$size)

biome.class.names.lpj <- c(
	"Tropical Rainforest",										# 1
	"Tropical Seasonal & Deciduous Forest",	# 2
	"Temperate Broadleaved Evergreen Forest",# 3
	"Temperate Broadleaved Deciduous Forest",# 4
	"Mixed Forest",													# 5
	"Temperate Coniferous Forest",						# 6
	"Boreal Evergreen Forest",								# 7
	"Boreal Deciduous Forest",								# 8
	"Warm Woody Savanna, Woodland & Shrubland",		# 9
	"Warm Savanna & Open Shrubland",							#10
	"Warm Grassland",															#11
	"Temperate Woody Savanna, Woodland & Shrubland",		# 12 "Temperate" instead of "Cool"
	"Temperate Savanna & Open Shrubland",							#13
	"Temperate Grassland",															#14
	"Arctic Tundra",													#15
	"Desert",																#16
	"Rocks and Ice",												#17
	"Water"																	#18
)
plot.classes <- c(1:16)

biome.class.cols.v2 <- c(rev(brewer.pal(9, "Oranges"))[2:3],
                         rev(brewer.pal(9, "YlGn"))[seq(1,7,by=2)],
                         rev(brewer.pal(9, "GnBu"))[4:3],
                         rev(brewer.pal(9, "Oranges"))[5:7],
                         rev(brewer.pal(9, "GnBu"))[c(6:8,1)],
                         "yellow")

models <- c(
	"cccma_cgcm3_1",    # 1
	"cnrm_cm3",         # 2
	"csiro_mk3_0",      # 3
	"gfdl_cm2_0",       # 4
	"gfdl_cm2_1",       # 5
	"giss_model_e_h",   # 6
	"giss_model_e_r",   # 7
	"iap_fgoals",       # 8
	"inmcm3_0",         # 9
	"ipsl_cm4",         #10
	"miroc3_2_hires",   #11
	"miroc3_2_medres",  #12
	"miub_echo_g",      #13
	"mpi_echam5",       #14
	"mri_cgcm2_3_2a",   #15
	"ncar_ccsm3_0",     #16
	"ncar_pcm1",        #17
	"ukmo_hadcm3",      #18
	"ukmo_hadgem1"      #19
)

temperatures <- seq(1.5, 5, by=0.5)


reference.grid.name <- paste(basepath,reference.path,"grid.out", sep="")
NCELL <- file.info(reference.grid.name)$size/4
grid.data <- array(readBin(reference.grid.name, what=integer(), size=2, n=NCELL*2), dim=c(2, NCELL))/100
earthradius <- 6371000.785
cellwidth <- earthradius*pi/180
grid.area <- (cellwidth*0.5)*(cellwidth*0.5)*cos(grid.data[2,]/180*pi)
natveg.frac <- readBin(paste(basepath, reference.path,"natvegfrac.bin", sep=""), what=double(), size=4, n=NCELL)

biome.statistics <- list()
if(file.exists(paste(basepath, "analysis/metric_components_v2.RData", sep="")) && !force_reload) {
  print(paste("Reloading biome statistics from ", basepath, "analysis/metric_components_v2.RData", sep=""))
	load(paste(basepath, "analysis/metric_components_v2.RData", sep=""))
}
for(subset in c("all", "cstocks", "cfluxes", "wfluxes")) {
  if(is.null(biome.statistics[[subset]]) || force_reload) {
    biome.statistics[[subset]] <- NULL
    biome.statistics[[subset]] <- list()
  }
  for(component in c("deltaV", "local", "global", "balance", "combined", "gamma")) {
    if(subset == "all" && component == "combined") next
    if(subset != "all" && component != "combined") next
    if(is.null(biome.statistics[[subset]][[component]]) || force_reload) {
      print(paste("**", subset, component, "**"))
      biome.statistics[[subset]][[component]] <- NULL
      biome.statistics[[subset]][[component]] <- list()
      load(paste(basepath, "analysis/", metric.filenames[[subset]][[component]], sep=""))
      biome.means <- array(dim=c(16,dim(data.array)[-1]))
      biome.lq <- array(dim=c(16,dim(data.array)[-1]))
      biome.uq <- array(dim=c(16,dim(data.array)[-1]))
      if(component %in% c("local", "global", "balance")) {
        SNratio.means <- array(dim=c(16,dim(data.array)[-1]))
        SNratio.lq <- array(dim=c(16,dim(data.array)[-1]))
        SNratio.uq <- array(dim=c(16,dim(data.array)[-1]))
        combined.means <- array(dim=c(16,dim(data.array)[-1]))
        combined.lq <- array(dim=c(16,dim(data.array)[-1]))
        combined.uq <- array(dim=c(16,dim(data.array)[-1]))
      }
      for(b in 1:16) {
        print(biome.class.names.lpj[b])
        biome.cells <- which(biome.lpj==b)
        biome.means[b,,] <- apply(data.array[biome.cells,,]*(grid.area*natveg.frac)[biome.cells], c(2,3), sum)/sum((grid.area*natveg.frac)[biome.cells])
        biome.lq[b,,] <- apply(data.array[biome.cells,,], c(2,3), wtd.quantile, probs=0.25, weights=(grid.area*natveg.frac)[biome.cells], type="quantile")
        biome.uq[b,,] <- apply(data.array[biome.cells,,], c(2,3), wtd.quantile, probs=0.75, weights=(grid.area*natveg.frac)[biome.cells], type="quantile")
        if(component %in% c("local", "global", "balance")) {
          SNratio.means[b,,] <- apply(ratio.array[biome.cells,,]*(grid.area*natveg.frac)[biome.cells], c(2,3), sum)/sum((grid.area*natveg.frac)[biome.cells])
          SNratio.lq[b,,] <- apply(ratio.array[biome.cells,,], c(2,3), wtd.quantile, probs=0.25, weights=(grid.area*natveg.frac)[biome.cells], type="quantile")
          SNratio.uq[b,,] <- apply(ratio.array[biome.cells,,], c(2,3), wtd.quantile, probs=0.75, weights=(grid.area*natveg.frac)[biome.cells], type="quantile")
          combined.means[b,,] <- apply(data.array[biome.cells,,]*ratio.array[biome.cells,,]*(grid.area*natveg.frac)[biome.cells],c(2,3), sum)/sum((grid.area*natveg.frac)[biome.cells])
          combined.lq[b,,] <- apply(data.array[biome.cells,,]*ratio.array[biome.cells,,], c(2,3), wtd.quantile, probs=0.25, weights=(grid.area*natveg.frac)[biome.cells], type="quantile")
          combined.uq[b,,] <- apply(data.array[biome.cells,,]*ratio.array[biome.cells,,], c(2,3), wtd.quantile, probs=0.75, weights=(grid.area*natveg.frac)[biome.cells], type="quantile")
        }
      }
      biome.statistics[[subset]][[component]][["mean"]] <- biome.means
      biome.statistics[[subset]][[component]][["lower_quantile"]] <- biome.lq
      biome.statistics[[subset]][[component]][["upper_quantile"]] <- biome.uq
      if(component %in% c("local", "global", "balance")) {
        biome.statistics[[subset]][[component]][["ratio_mean"]] <- SNratio.means
        biome.statistics[[subset]][[component]][["ratio_lower_quantile"]] <- SNratio.lq
        biome.statistics[[subset]][[component]][["ratio_upper_quantile"]] <- SNratio.uq
        biome.statistics[[subset]][[component]][["combined_mean"]] <- combined.means
        biome.statistics[[subset]][[component]][["combined_lower_quantile"]] <- combined.lq
        biome.statistics[[subset]][[component]][["combined_upper_quantile"]] <- combined.uq
      }
    } else {
      print(paste("**", subset, component, "skipped **"))
    }
  }
}
if(save_statistics) {
  save(biome.statistics, file=paste(basepath, "analysis/metric_components_v2.RData", sep=""))
}

name_dimensions <- function(inobject) {
  if(!is.array(inobject)) {
    stop("Error: object is not an array")
  }
  if(toString(dim(inobject)) != toString(c(length(plot.classes), length(models), length(temperatures)))) {
    stop("Error: array has wrong dimensions")
  }
  dimnames(inobject)[[1]] <- biome.class.names.lpj[plot.classes]
  dimnames(inobject)[[2]] <- models
  dimnames(inobject)[[3]] <- paste(temperatures, "K", sep="")
  return(inobject)
}
biome.statistics <- rapply(object=biome.statistics, f=name_dimensions, how="replace")

pie.data <- array(dim=c(16,11,3,8))
pie.data2 <- array(dim=c(16,11,3,8))
i <- 1
j <- 1
dimnames(pie.data)[[1]] <- biome.class.names.lpj[1:16]
dimnames(pie.data2)[[1]] <- biome.class.names.lpj[1:16]

median.model <- NULL
median_index <- function(values) {
 index <- which(values==median(values))
 if(length(index) != 1) {
  warning(paste("Warning: Median index could not be determined conclusively. (", length(index), "values )"))
 }
 return(index[1])
}
median.model <- apply(biome.statistics[["all"]][["gamma"]][["mean"]][,,], c(1,3), median_index)

load(paste(basepath, "analysis/", metric.filenames[["all"]][["gamma"]], sep=""))
biome.area.severe <- array(dim=c(16,19,8))
severe.data <- ifelse(data.array>=0.3, 1, 0)
severe.nmodels <- apply(severe.data, c(1,3), sum)
biome.area.severe_atleast1 <- array(dim=c(16,8))

biome.area.moderate <- array(dim=c(16,19,8))
moderate.data <- ifelse(data.array>=0.1, 1, 0)
moderate.nmodels <- apply(moderate.data, c(1,3), sum)
biome.area.moderate_atleast1 <- array(dim=c(16,8))

for(subset in c("all", "cstocks", "cfluxes", "wfluxes")) {
  for(component in c("gamma", "deltaV", "local", "global", "balance", "combined")) {
		if(!is.null(biome.statistics[[subset]][[component]])) {
			for(b in 1:16) {
        biome.cells <- which(biome.lpj==b)
        biome.area <- sum((grid.area*natveg.frac)[biome.cells])
        biome.gamma <- data.array[biome.cells,,]
			  for(t in 1:8) {
			    pie.data[b,i,1,t] <- biome.statistics[[subset]][[component]][["mean"]][b,median.model[b,t],t] 
  		    pie.data2[b,j,1,t] <- biome.statistics[[subset]][[component]][["mean"]][b,median.model[b,t],t]
  		    if(component=="gamma") {
  		      for(m in 1:19) {
  		        biome.area.severe[b,m,t] <- sum(((grid.area*natveg.frac)[biome.cells])[which(biome.gamma[,m,t]>=0.3)])/biome.area
  		        biome.area.moderate[b,m,t] <- sum(((grid.area*natveg.frac)[biome.cells])[which(biome.gamma[,m,t]>=0.1)])/biome.area
  		      }
  		      biome.area.severe_atleast1[b,t] <- sum(((grid.area*natveg.frac)[biome.cells])[which(severe.nmodels[biome.cells,t]>0)])/biome.area
  		      biome.area.moderate_atleast1[b,t] <- sum(((grid.area*natveg.frac)[biome.cells])[which(moderate.nmodels[biome.cells,t]>0)])/biome.area
  		    }
		    }
	    }
	    pie.data[,i,2,] <- apply(biome.statistics[[subset]][[component]][["mean"]][,,], c(1,3), min)
	    pie.data[,i,3,] <- apply(biome.statistics[[subset]][[component]][["mean"]][,,], c(1,3), max)
	    pie.data2[,j,2,] <- apply(biome.statistics[[subset]][[component]][["mean"]][,,], c(1,3), min)
	    pie.data2[,j,3,] <- apply(biome.statistics[[subset]][[component]][["mean"]][,,], c(1,3), max)
	    i <- i+1
	    j <- j+1
	    if(component %in% c("local", "global", "balance")) {
  			for(b in 1:16) {
  			  for(t in 1:8) {
    		    pie.data[b,i,1,t] <-  biome.statistics[[subset]][[component]][["ratio_mean"]][b,median.model[b,t],t] 
       			pie.data2[b,j,1,t] <- biome.statistics[[subset]][[component]][["combined_mean"]][b,median.model[b,t],t]
     			}
   			}
		    pie.data[,i,2,] <-  apply(biome.statistics[[subset]][[component]][["ratio_mean"]][,,], c(1,3), min)
		    pie.data[,i,3,] <-  apply(biome.statistics[[subset]][[component]][["ratio_mean"]][,,], c(1,3), max)
   			pie.data2[,j,2,] <- apply(biome.statistics[[subset]][[component]][["combined_mean"]][,,], c(1,3), min)
   			pie.data2[,j,3,] <- apply(biome.statistics[[subset]][[component]][["combined_mean"]][,,], c(1,3), max)
  	    i<-i+1
   			j<-j+1
	    }
		}
	}
}

temperature.levels <- c(2,5,8)
level.names <- c("2 degree target", "current emissions reduction pledges (3.5 K warming)", "business-as-usual emissions growth (5 K warming)")

radius_based <- FALSE

variable.names <- c(expression(Gamma), expression(paste(Delta, "V")), "c", expression(paste("S(c,", sigma[c], ")")), "g", expression(paste("S(g,", sigma[g], ")")), "b", expression(paste("S(b,", sigma[b], ")")), "carbon stocks", "carbon fluxes", "water fluxes")
variable.names2 <- c(expression(paste(Gamma, " metric")), "vegetation\nstructure", "c", "local\nchange", "g", "global\nimportance", "b", "ecosystem\nbalance", "carbon stocks", "carbon fluxes", "water fluxes")
variable.cols <- c("darkgoldenrod", brewer.pal(5,"Greens")[5], brewer.pal(6, "Set1"), rev(brewer.pal(6, "Oranges")[c(4,5)]), brewer.pal(6, "PuBu")[6])
variable.order <- c(NA, 11:9, NA, 8:1) # order of metric components
variable.order2 <- c(NA, 11:9, NA, 8,6,4,2,1)

dimnames(pie.data)[[2]] <- variable.names
dimnames(pie.data2)[[2]] <- variable.names2
dimnames(pie.data)[[3]] <- c("median", "min", "max")
dimnames(pie.data2)[[3]] <- c("median", "min", "max")
dimnames(pie.data)[[4]] <- paste(temperatures, "K", sep="")
dimnames(pie.data2)[[4]] <- paste(temperatures, "K", sep="")



display.order <- c(1:2, 9:11, 16, 3:6, 12:14, 7:8, 15) # biomes order
labels <- unlist(lapply(strwrap(biome.class.names.lpj[display.order], 20, simplify = F), paste, sep="", collapse="\n"))

stop("Stop automatic run")

xpos <- c(seq(1.5, by=3, length.out=6), seq(1.5, by=3, length.out=7), seq(1.5, by=3, length.out=3))
ypos <- c(rep(8, 6), rep(5, 7), rep(2, 3)) 
xlim <- range(xpos)+c(-1.5,1.5)
ylim <- range(ypos)+c(-1.5,1)


# Version 3: reordering, condensed variables
# used in SI, revised version
deg1 <- seq(0,360, length.out=length(variable.order2)+1)
deg2 <- c(deg1[-1], deg1[1])
deg1 <- deg1+(deg1[2]-deg1[1])/4#5
deg2 <- deg2-(deg1[2]-deg1[1])/4
setEPS()
if(radius_based) {
	plot.data <- pie.data2
} else {
	plot.data <- sqrt(pie.data2)
}

xpos <- seq(1.5, by=3, length.out=length(temperature.levels))
xpos <- rep(xpos, each=8)
dim(xpos) <- c(8,length(temperature.levels))
ypos <- rev(seq(1.5, by=2.25, length.out=8))
ypos <- rep(ypos, length(temperature.levels))
dim(ypos) <- c(8,length(temperature.levels))

xlim <- range(xpos)+c(-1.5,7.5)
ylim <- range(ypos)+c(-1,1)

for(page in 1:2) {
  display.page <- display.order[1:8+(page-1)*8]
  postscript(paste(basepath, "analysis/metric_components", ifelse(radius_based, "_radius", "_area"), "_reduced_p",page, "_v3.eps", sep=""), width=503/72, height=660/72, pointsize=8, paper="special")
  par(mar=c(0,0,3,0)+0.1,cex=1)
  plot(1,1, type="n", axes=FALSE, xlim=xlim, ylim=ylim, asp=1, xaxs="i", yaxs="i")
  for(p in 1:length(temperature.levels)) {
	  if(p==length(temperature.levels)) {
		  key.loc <- c(max(xpos)+5.5, min(ypos))
	  } else {
		  key.loc <- NULL
	  }
	  stars(plot.data[display.page,variable.order2,1,temperature.levels[p]], draw.segments=T, scale=F, key.loc=key.loc, key.labels=NULL, add=T, locations=cbind(xpos[,p], ypos[,p]), col.segments=variable.cols[variable.order2], labels=NULL, len=1)#key.labels=c(variable.names2[variable.order2])
	  for(i in 1:length(variable.order2)) {
		  if(!is.na(variable.order2[i])) {
			  draw.arc(cbind(xpos[,p], ypos[,p]), radius=plot.data[display.page,variable.order2[i],2,temperature.levels[p]], deg1=deg1[i], deg2=deg2[i], lwd=0.5)
			  draw.arc(cbind(xpos[,p], ypos[,p]), radius=plot.data[display.page,variable.order2[i],3,temperature.levels[p]], deg1=deg1[i], deg2=deg2[i], lwd=0.5)
    	  if(p==length(temperature.levels)) { 
          pie.labels(x=key.loc[1], y=key.loc[2], angles=(mean(c(deg1[i],deg1[i+1]))-5)/180*pi, labels=variable.names2[variable.order2[i]],
                     radius=ifelse((mean(c(deg1[i],deg1[i+1]))-5)>260 & grepl("\n",variable.names2[variable.order2[i]]) |(mean(c(deg1[i],deg1[i+1]))-5)<165, 1.3,1.1),
                     border=FALSE, col="black",
                     adj=c(ifelse(cos(deg1[i]/180*pi)>0, abs(sin(deg1[i]/180*pi)^10)*0.5, 1-(abs(sin(deg1[i]/180*pi))*0.5)^2), 1),
                     cex=0.8, bg="transparent")
        }
			  for(b in 1:8) {
				  draw.radial.line(start=plot.data[display.page[b],variable.order2[i],2,temperature.levels[p]],end=plot.data[display.page[b],variable.order2[i],3,temperature.levels[p]],center=c(xpos[b,p],ypos[b,p]), deg=mean(c(deg1[i],deg2[i]))) 
			  }
		  }
	  }
	  if(p==length(temperature.levels)) { 
	    text(cbind(xpos[,p]+1.5, ypos[,p]), labels=labels[1:8+(page-1)*8], adj=c(0,0.5), cex=0.8)
	  }
	  symbols(cbind(xpos[,p], ypos[,p]), circles=rep(1,length(display.page)), add=T, inches=FALSE,lwd=0.5, col="grey50")
	  symbols(cbind(xpos[,p], ypos[,p]), circles=rep(ifelse(radius_based, 0.5, sqrt(0.5)),length(display.page)), add=T, inches=FALSE, lwd=0.25, col="grey50")
	  symbols(cbind(xpos[,p], ypos[,p]), circles=rep(ifelse(radius_based, 0.25, sqrt(0.25)),length(display.page)), add=T, inches=FALSE, lwd=0.15, col="grey50")

    if(p==length(temperature.levels)) {
  	  symbols(cbind(key.loc[1], ypos[6,p]), circles=rep(1,1), add=T, inches=FALSE,lwd=0.5, col="grey50")
	    symbols(cbind(key.loc[1], ypos[6,p]), circles=rep(ifelse(radius_based, 0.5, sqrt(0.5)),1), add=T, inches=FALSE, lwd=0.25, col="grey50")
	    symbols(cbind(key.loc[1], ypos[6,p]), circles=rep(ifelse(radius_based, 0.25, sqrt(0.25)),1), add=T, inches=FALSE, lwd=0.15, col="grey50")
		  draw.radial.line(start=ifelse(radius_based, 0.25, sqrt(0.25)), end=1.25, center=c(key.loc[1], ypos[6,p]), deg=0, lwd=0.5, lty=3)
		  draw.radial.line(start=ifelse(radius_based, 0.5, sqrt(0.5)), end=1.25, center=c(key.loc[1], ypos[6,p]), deg=10, lwd=0.5, lty=3)
		  draw.radial.line(start=1, end=1.25, center=c(key.loc[1], ypos[6,p]), deg=20, lwd=0.5, lty=3)
		  draw.arc(cbind(key.loc[1], ypos[6,p]), radius=ifelse(radius_based, 0.4, sqrt(0.4)), deg1=-18+90, deg2=18+90, lwd=0.25)
		  draw.radial.line(start=0, end=ifelse(radius_based, 0.4, sqrt(0.4)), center=c(key.loc[1], ypos[6,p]), deg=c(-18)+90, lwd=0.25)
		  draw.radial.line(start=0, end=ifelse(radius_based, 0.4, sqrt(0.4)), center=c(key.loc[1], ypos[6,p]), deg=c(18)+90, lwd=0.25)
		  pie.labels(x=key.loc[1], y=ypos[6,p], angles=c(0,10,20)/180*pi, labels=c(0.25,0.5,1), radius=1.25, adj=0, cex=0.8, border=FALSE, bg="transparent")
		  draw.arc(cbind(key.loc[1], ypos[6,p]), radius=ifelse(radius_based, 0.3, sqrt(0.3)), deg1=-9+90, deg2=9+90, lwd=0.5)
		  draw.arc(cbind(key.loc[1], ypos[6,p]), radius=ifelse(radius_based, 0.6, sqrt(0.6)), deg1=-9+90, deg2=9+90, lwd=0.5)
		  draw.radial.line(start=ifelse(radius_based, 0.3, sqrt(0.3)), end=ifelse(radius_based, 0.6, sqrt(0.6)), center=c(key.loc[1], ypos[6,p]), deg=90)
		  points(x=key.loc[1], y=ypos[6,p], pch=3, cex=0.8)
    }
  }
  rect(xleft=0, ybottom=ypos[,1]-1.1, xright=0.25, ytop=ypos[,1]+1.1, density=NA, col=biome.class.cols.v2[display.page])
  rect(xleft=0, ybottom=ypos[,1]-1.1, xright=max(xpos)+3.5, ytop=ypos[,1]+1.1, density=NULL, border=biome.class.cols.v2[display.page])
  mtext(text=unlist(lapply(strwrap(level.names, 18, simplify = F), paste, sep="", collapse="\n")), side=3,
        line=2-str_count(unlist(lapply(strwrap(level.names, 18, simplify = F), paste, sep="", collapse="\n")), "\n"),
        at=xpos[1,], adj=0.5, padj=0)
  dev.off()
}

# version 3.5, same as 3, but only 4 biomes
# used in main text, revised version
deg1 <- seq(0,360, length.out=length(variable.order2)+1)
deg2 <- c(deg1[-1], deg1[1])
deg1 <- deg1+(deg1[2]-deg1[1])/4#5
deg2 <- deg2-(deg1[2]-deg1[1])/4
setEPS()
if(radius_based) {
	plot.data <- pie.data2
} else {
	plot.data <- sqrt(pie.data2)
}

display.page <- display.order[c(1, 13,14,16)]#display.order[1:8+(page-1)*8]
labels.sub <- labels[c(1, 13,14,16)]
xpos <- seq(1.5, by=3, length.out=length(temperature.levels))
xpos <- rep(xpos, each=length(display.page))
dim(xpos) <- c(length(display.page),length(temperature.levels))
ypos <- rev(seq(1.5, by=2.25, length.out=length(display.page)))
ypos <- rep(ypos, length(temperature.levels))
dim(ypos) <- c(length(display.page),length(temperature.levels))

xlim <- range(xpos)+c(-1.5,7.5)
ylim <- range(ypos)+c(-1,1)

  postscript(paste(basepath, "analysis/metric_components", ifelse(radius_based, "_radius", "_area"), "_reduced_maintext.eps", sep=""), width=503/72, height=660/72*5/9, pointsize=8, paper="special")
  par(mar=c(0,0,3,0)+0.1,cex=1)
  plot(1,1, type="n", axes=FALSE, xlim=xlim, ylim=ylim, asp=1, xaxs="i", yaxs="i")
  for(p in 1:length(temperature.levels)) {
	  if(p==length(temperature.levels)) {
		  key.loc <- c(max(xpos)+5.5, min(ypos))
	  } else {
		  key.loc <- NULL
	  }
	  stars(plot.data[display.page,variable.order2,1,temperature.levels[p]], draw.segments=T, scale=F, key.loc=key.loc, key.labels=NULL, add=T, locations=cbind(xpos[,p], ypos[,p]), col.segments=variable.cols[variable.order2], labels=NULL, len=1)#key.labels=c(variable.names2[variable.order2])
	  for(i in 1:length(variable.order2)) {
		  if(!is.na(variable.order2[i])) {
			  draw.arc(cbind(xpos[,p], ypos[,p]), radius=plot.data[display.page,variable.order2[i],2,temperature.levels[p]], deg1=deg1[i], deg2=deg2[i], lwd=0.5)
			  draw.arc(cbind(xpos[,p], ypos[,p]), radius=plot.data[display.page,variable.order2[i],3,temperature.levels[p]], deg1=deg1[i], deg2=deg2[i], lwd=0.5)
    	  if(p==length(temperature.levels)) { 
          pie.labels(x=key.loc[1], y=key.loc[2], angles=(mean(c(deg1[i],deg1[i+1]))-5)/180*pi, labels=variable.names2[variable.order2[i]],
                     radius=ifelse((mean(c(deg1[i],deg1[i+1]))-5)>260 & grepl("\n",variable.names2[variable.order2[i]]) |(mean(c(deg1[i],deg1[i+1]))-5)<165, 1.3,1.1),
                     border=FALSE, col="black",
                     adj=c(ifelse(cos(deg1[i]/180*pi)>0, abs(sin(deg1[i]/180*pi)^10)*0.5, 1-(abs(sin(deg1[i]/180*pi))*0.5)^2), 1),
                     cex=0.8, bg="transparent")
        }
			  for(b in 1:length(display.page)) {
				  draw.radial.line(start=plot.data[display.page[b],variable.order2[i],2,temperature.levels[p]],end=plot.data[display.page[b],variable.order2[i],3,temperature.levels[p]],center=c(xpos[b,p],ypos[b,p]), deg=mean(c(deg1[i],deg2[i]))) 
			  }
		  }
	  }
	  if(p==length(temperature.levels)) { 
	    text(cbind(xpos[,p]+1.5, ypos[,p]), labels=labels.sub, adj=c(0,0.5), cex=0.8)
	  }
	  symbols(cbind(xpos[,p], ypos[,p]), circles=rep(1,length(display.page)), add=T, inches=FALSE,lwd=0.5, col="grey50")
	  symbols(cbind(xpos[,p], ypos[,p]), circles=rep(ifelse(radius_based, 0.5, sqrt(0.5)),length(display.page)), add=T, inches=FALSE, lwd=0.25, col="grey50")
	  symbols(cbind(xpos[,p], ypos[,p]), circles=rep(ifelse(radius_based, 0.25, sqrt(0.25)),length(display.page)), add=T, inches=FALSE, lwd=0.15, col="grey50")

    if(p==length(temperature.levels)) {
  	  symbols(cbind(key.loc[1], ypos[2,p]), circles=rep(1,1), add=T, inches=FALSE,lwd=0.5, col="grey50")
	    symbols(cbind(key.loc[1], ypos[2,p]), circles=rep(ifelse(radius_based, 0.5, sqrt(0.5)),1), add=T, inches=FALSE, lwd=0.25, col="grey50")
	    symbols(cbind(key.loc[1], ypos[2,p]), circles=rep(ifelse(radius_based, 0.25, sqrt(0.25)),1), add=T, inches=FALSE, lwd=0.15, col="grey50")
		  draw.radial.line(start=ifelse(radius_based, 0.25, sqrt(0.25)), end=1.25, center=c(key.loc[1], ypos[2,p]), deg=0, lwd=0.5, lty=3)
		  draw.radial.line(start=ifelse(radius_based, 0.5, sqrt(0.5)), end=1.25, center=c(key.loc[1], ypos[2,p]), deg=10, lwd=0.5, lty=3)
		  draw.radial.line(start=1, end=1.25, center=c(key.loc[1], ypos[2,p]), deg=20, lwd=0.5, lty=3)
		  draw.arc(cbind(key.loc[1], ypos[2,p]), radius=ifelse(radius_based, 0.4, sqrt(0.4)), deg1=-18+90, deg2=18+90, lwd=0.25)
		  draw.radial.line(start=0, end=ifelse(radius_based, 0.4, sqrt(0.4)), center=c(key.loc[1], ypos[2,p]), deg=c(-18)+90, lwd=0.25)
		  draw.radial.line(start=0, end=ifelse(radius_based, 0.4, sqrt(0.4)), center=c(key.loc[1], ypos[2,p]), deg=c(18)+90, lwd=0.25)
		  pie.labels(x=key.loc[1], y=ypos[2,p], angles=c(0,10,20)/180*pi, labels=c(0.25,0.5,1), radius=1.25, adj=0, cex=0.8, border=FALSE, bg="transparent")
		  draw.arc(cbind(key.loc[1], ypos[2,p]), radius=ifelse(radius_based, 0.3, sqrt(0.3)), deg1=-9+90, deg2=9+90, lwd=0.5)
		  draw.arc(cbind(key.loc[1], ypos[2,p]), radius=ifelse(radius_based, 0.6, sqrt(0.6)), deg1=-9+90, deg2=9+90, lwd=0.5)
		  draw.radial.line(start=ifelse(radius_based, 0.3, sqrt(0.3)), end=ifelse(radius_based, 0.6, sqrt(0.6)), center=c(key.loc[1], ypos[2,p]), deg=90)
		  points(x=key.loc[1], y=ypos[2,p], pch=3, cex=0.8)
    }
  }
  rect(xleft=0, ybottom=ypos[,1]-1.1, xright=0.25, ytop=ypos[,1]+1.1, density=NA, col=biome.class.cols.v2[display.page])
  rect(xleft=0, ybottom=ypos[,1]-1.1, xright=max(xpos)+3.5, ytop=ypos[,1]+1.1, density=NULL, border=biome.class.cols.v2[display.page])
  mtext(text=unlist(lapply(strwrap(level.names, 18, simplify = F), paste, sep="", collapse="\n")), side=3,
        line=2-str_count(unlist(lapply(strwrap(level.names, 18, simplify = F), paste, sep="", collapse="\n")), "\n"),
        at=xpos[1,], adj=0.5, padj=0)
  dev.off()
